@@ -206,8 +206,18 @@ if has_cextensions:
 class CompilePython(Compile):
 
     def get_matcher(self, expr):
-        exec "def match(get_column): return bool(%s)" % self(expr)
-        return match
+        state = State()
+        source = self(expr, state)
+        namespace = {}
+        code = ("def closure(parameters, bool):\n"
+                "    [%s] = parameters\n"
+                "    def match(get_column):\n"
+                "        return bool(%s)\n"
+                "    return match" %
+                (",".join("_%d" % i for i in range(len(state.parameters))),
+                 source))
+        exec code in namespace
+        return namespace['closure'](state.parameters, bool)
 
 
 class State(object):
@@ -348,12 +358,18 @@ def compile_none(compile, expr, state):
     return "NULL"
 
 
-@compile_python.when(str, unicode, bool, int, long, float,
-                     datetime, date, time, timedelta, type(None))
+@compile_python.when(str, unicode, int, long, float, type(None))
 def compile_python_builtin(compile, expr, state):
     return repr(expr)
 
 
+@compile_python.when(bool, datetime, date, time, timedelta)
+def compile_python_bool_and_dates(compile, expr, state):
+    index = len(state.parameters)
+    state.parameters.append(expr)
+    return "_%d" % index
+
+
 @compile.when(Variable)
 def compile_variable(compile, variable, state):
     state.parameters.append(variable)
@@ -361,7 +377,9 @@ def compile_variable(compile, variable, state):
 
 @compile_python.when(Variable)
 def compile_python_variable(compile, variable, state):
-    return repr(variable.get())
+    index = len(state.parameters)
+    state.parameters.append(variable.get())
+    return "_%d" % index
 
 
 # --------------------------------------------------------------------
@@ -377,6 +395,15 @@ def compile_python_unsupported(compile, expr, state):
     raise CompileError("Can't compile python expressions with %r" % type(expr))
 
 
+# A translation table that can escape a unicode string for use in a
+# Like() expression that uses "!" as the escape character.
+like_escape = {
+    ord(u"!"): u"!!",
+    ord(u"_"): u"!_",
+    ord(u"%"): u"!%"
+    }
+
+
 class Comparable(object):
     __slots__ = ()
 
@@ -455,6 +482,9 @@ class Comparable(object):
             other = getattr(self, "variable_factory", Variable)(value=other)
         return Mod(self, other)
 
+    def __neg__(self):
+        return Neg(self)
+
     def is_in(self, others):
         if not isinstance(others, Expr):
             others = list(others)
@@ -477,6 +507,24 @@ class Comparable(object):
     def upper(self):
         return Upper(self)
 
+    def startswith(self, prefix):
+        if not isinstance(prefix, unicode):
+            raise ExprError("Expected unicode argument, got %r" % type(prefix))
+        pattern = prefix.translate(like_escape) + u"%"
+        return Like(self, pattern, u"!")
+
+    def endswith(self, suffix):
+        if not isinstance(suffix, unicode):
+            raise ExprError("Expected unicode argument, got %r" % type(suffix))
+        pattern = u"%" + suffix.translate(like_escape)
+        return Like(self, pattern, u"!")
+
+    def contains_string(self, substring):
+        if not isinstance(substring, unicode):
+            raise ExprError("Expected unicode argument, got %r" % type(substring))
+        pattern = u"%" + substring.translate(like_escape) + u"%"
+        return Like(self, pattern, u"!")
+
 
 class ComparableExpr(Expr, Comparable):
     __slots__ = ()
@@ -776,7 +824,9 @@ def compile_column(compile, column, state):
 
 @compile_python.when(Column)
 def compile_python_column(compile, column, state):
-    return "get_column(%s)" % repr(column.name)
+    index = len(state.parameters)
+    state.parameters.append(column)
+    return "get_column(_%d)" % index
 
 
 # --------------------------------------------------------------------
@@ -1076,6 +1126,16 @@ class SetExpr(Expr):
         self.order_by = kwargs.get("order_by", Undef)
         self.limit = kwargs.get("limit", Undef)
         self.offset = kwargs.get("offset", Undef)
+        # If the first expression is of a compatible type, directly
+        # include its sub expressions.
+        if len(self.exprs) > 0:
+            first = self.exprs[0]
+            if (isinstance(first, self.__class__) and
+                first.all == self.all and
+                first.limit is Undef and
+                first.offset is Undef):
+                self.exprs = first.exprs + self.exprs[1:]
+
 
 @compile.when(SetExpr)
 def compile_set_expr(compile, expr, state):
@@ -1165,9 +1225,12 @@ class Count(FuncExpr):
 @compile.when(Count)
 def compile_count(compile, count, state):
     if count.column is not Undef:
+        state.push("context", EXPR)
+        column = compile(count.column, state)
+        state.pop()
         if count.distinct:
-            return "COUNT(DISTINCT %s)" % compile(count.column, state)
-        return "COUNT(%s)" % compile(count.column, state)
+            return "COUNT(DISTINCT %s)" % column
+        return "COUNT(%s)" % column
     return "COUNT(*)"
 
 
@@ -1186,7 +1249,10 @@ class NamedFunc(FuncExpr):
 
 @compile.when(Func, NamedFunc)
 def compile_func(compile, func, state):
-    return "%s(%s)" % (func.name, compile(func.args, state))
+    state.push("context", EXPR)
+    args = compile(func.args, state)
+    state.pop()
+    return "%s(%s)" % (func.name, args)
 
 
 class Max(NamedFunc):
@@ -1215,6 +1281,11 @@ class Upper(NamedFunc):
     name = "UPPER"
 
 
+class Coalesce(NamedFunc):
+    __slots__ = ()
+    name = "COALESCE"
+
+
 # --------------------------------------------------------------------
 # Prefix and suffix expressions
 
@@ -1250,6 +1321,14 @@ class Exists(PrefixExpr):
     __slots__ = ()
     prefix = "EXISTS"
 
+class Neg(PrefixExpr):
+    __slots__ = ()
+    prefix = "-"
+
+@compile_python.when(Neg)
+def compile_neg_expr(compile, expr, state):
+    return "-%s" % compile(expr.expr, state, raw=True)
+
 class Asc(SuffixExpr):
     __slots__ = ()
     suffix = "ASC"
