@@ -68,6 +68,7 @@ class Store(object):
         @param database: The L{storm.database.Database} instance to use.
         @param cache: The cache to use.  Defaults to a L{Cache} instance.
         """
+        self._database = database
         self._event = EventSystem(self)
         self._connection = database.connect(self._event)
         self._alive = WeakValueDictionary()
@@ -80,6 +81,10 @@ class Store(object):
         self._implicit_flush_block_count = 0
         self._sequence = 0 # Advisory ordering.
 
+    def get_database(self):
+        """Return this Store's Database object."""
+        return self._database
+
     @staticmethod
     def of(obj):
         """Get the Store that the object is associated with.
@@ -309,8 +314,8 @@ class Store(object):
                         default_tables=cls_info.table, limit=1)
         result = self._connection.execute(select)
         values = result.get_one()
-        self._set_values(obj_info, cls_info.columns, result, values)
-        obj_info.checkpoint()
+        self._set_values(obj_info, cls_info.columns, result, values,
+                         replace_unknown_lazy=True)
         self._set_clean(obj_info)
 
     def autoreload(self, obj=None):
@@ -529,13 +534,8 @@ class Store(object):
 
             self._enable_change_notification(obj_info)
             self._add_to_alive(obj_info)
-
-            obj_info.checkpoint()
-
         else:
-
             cached_primary_vars = obj_info["primary_vars"]
-            primary_key_idx = cls_info.primary_key_idx
 
             changes = self._get_changes_map(obj_info)
 
@@ -550,9 +550,6 @@ class Store(object):
 
                 self._add_to_alive(obj_info)
 
-
-            obj_info.checkpoint()
-
         self._run_hook(obj_info, "__storm_flushed__")
 
         obj_info.event.emit("flushed")
@@ -566,6 +563,14 @@ class Store(object):
         assert self._implicit_flush_block_count > 0
         self._implicit_flush_block_count -= 1
 
+    def block_access(self):
+        """Block access to the underlying database connection."""
+        self._connection.block_access()
+
+    def unblock_access(self):
+        """Unblock access to the underlying database connection."""
+        self._connection.unblock_access()
+
     def _get_changes_map(self, obj_info, adding=False):
         """Return a {column: variable} dictionary suitable for inserts/updates.
 
@@ -640,6 +645,8 @@ class Store(object):
                     # must have just been sent to the database, so this
                     # was already set there.
                     variable.set(AutoReload)
+            else:
+                variable.checkpoint()
 
         if missing_columns:
             where = result.get_insert_identity(cls_info.primary_key,
@@ -690,14 +697,14 @@ class Store(object):
             # primary key was extracted from result values.
             obj_info.pop("invalidated", None)
 
-            # We're not sure if the obj is still in memory at this
-            # point.  This will rebuild it if needed.
-            obj = self._get_object(obj_info)
-
             # Take that chance and fill up any undefined variables
             # with fresh data, since we got it anyway.
             self._set_values(obj_info, cls_info.columns, result,
                              values, keep_defined=True)
+
+            # We're not sure if the obj is still in memory at this
+            # point.  This will rebuild it if needed.
+            obj = self._get_object(obj_info)
         else:
             # Nothing found in the cache. Build everything from the ground.
             obj = cls.__new__(cls)
@@ -705,9 +712,8 @@ class Store(object):
             obj_info = get_obj_info(obj)
             obj_info["store"] = self
 
-            self._set_values(obj_info, cls_info.columns, result, values)
-
-            obj_info.checkpoint()
+            self._set_values(obj_info, cls_info.columns, result, values,
+                             replace_unknown_lazy=True)
 
             self._add_to_alive(obj_info)
             self._enable_change_notification(obj_info)
@@ -740,31 +746,35 @@ class Store(object):
             func()
 
     def _set_values(self, obj_info, columns, result, values,
-                    keep_defined=False):
+                    keep_defined=False, replace_unknown_lazy=False):
         if values is None:
             raise LostObjectError("Can't obtain values from the database "
                                   "(object got removed?)")
         obj_info.pop("invalidated", None)
         for column, value in zip(columns, values):
             variable = obj_info.variables[column]
+            lazy_value = variable.get_lazy()
+            is_unknown_lazy = not (lazy_value is None or
+                                   lazy_value is AutoReload)
             if keep_defined:
-                if variable.is_defined():
+                if variable.is_defined() or is_unknown_lazy:
                     continue
-                lazy_value = variable.get_lazy()
-                if not (lazy_value is None or lazy_value is AutoReload):
-                    # This should *never* happen, because whenever we get
-                    # to this point it should be after a flush() which
-                    # updated the database with lazy values and then replaced
-                    # them by AutoReload.  Letting this go through means
-                    # we're blindly discarding an unknown lazy value and
-                    # replacing it by the value from the database.
-                    raise RuntimeError("Unexpected situation. "
-                                       "Please contact the developers.")
+            elif is_unknown_lazy and not replace_unknown_lazy:
+                # This should *never* happen, because whenever we get
+                # to this point it should be after a flush() which
+                # updated the database with lazy values and then replaced
+                # them by AutoReload.  Letting this go through means
+                # we're blindly discarding an unknown lazy value and
+                # replacing it by the value from the database.
+                raise RuntimeError("Unexpected situation. "
+                                   "Please contact the developers.")
             if value is None:
                 variable.set(value, from_db=True)
             else:
                 result.set_variable(variable, value)
 
+            variable.checkpoint()
+
 
     def _is_dirty(self, obj_info):
         return obj_info in self._dirty
@@ -885,11 +895,10 @@ class Store(object):
         if autoreload_columns:
             where = compare_columns(obj_info.cls_info.primary_key,
                                     obj_info["primary_vars"])
-            result = self._connection.execute(Select(autoreload_columns, where))
+            result = self._connection.execute(
+                Select(autoreload_columns, where))
             self._set_values(obj_info, autoreload_columns,
                              result, result.get_one())
-            for column in autoreload_columns:
-                obj_info.variables[column].checkpoint()
 
 
 class ResultSet(object):
@@ -902,6 +911,7 @@ class ResultSet(object):
     Generally these should not be constructed directly, but instead
     retrieved from calls to L{Store.find}.
     """
+
     def __init__(self, store, find_spec,
                  where=Undef, tables=Undef, select=Undef):
         self._store = store
@@ -921,6 +931,10 @@ class ResultSet(object):
         """
         result_set = object.__new__(self.__class__)
         result_set.__dict__.update(self.__dict__)
+        if self._select is not Undef:
+            # This expression must be copied because we may have to change it
+            # in-place inside _get_select().
+            result_set._select = copy(self._select)
         return result_set
 
     def config(self, distinct=None, offset=None, limit=None):
@@ -972,8 +986,9 @@ class ResultSet(object):
     def __getitem__(self, index):
         """Get an individual item by offset, or a range of items by slice.
 
-        If a slice is used, a new L{ResultSet} will be return
-        appropriately modified with OFFSET and LIMIT clauses.
+        @return: The matching object or, if a slice is used, a new
+            L{ResultSet} will be returned appropriately modified with
+            C{OFFSET} and C{LIMIT} clauses.
         """
         if isinstance(index, (int, long)):
             if index == 0:
@@ -983,7 +998,7 @@ class ResultSet(object):
                     index += self._offset
                 result_set = self.copy()
                 result_set.config(offset=index, limit=1)
-            obj = result_set.any()
+            obj = result_set._any()
             if obj is None:
                 raise IndexError("Index out of range")
             return obj
@@ -1039,9 +1054,10 @@ class ResultSet(object):
         return result.get_one() is not None
 
     def is_empty(self):
-        """Return true if this L{ResultSet} contains no results."""
+        """Return C{True} if this result set doesn't contain any results."""
         subselect = self._get_select()
         subselect.limit = 1
+        subselect.order_by = Undef
         select = Select(1, tables=Alias(subselect, "_tmp"), limit=1)
         result = self._store._connection.execute(select)
         return (not result.get_one())
@@ -1049,7 +1065,22 @@ class ResultSet(object):
     def any(self):
         """Return a single item from the result set.
 
-        See also one(), first(), and last().
+        @return: An arbitrary object or C{None} if one isn't available.
+        @seealso: one(), first(), and last().
+        """
+        select = self._get_select()
+        select.limit = 1
+        select.order_by = Undef
+        result = self._store._connection.execute(select)
+        values = result.get_one()
+        if values:
+            return self._load_objects(result, values)
+        return None
+
+    def _any(self):
+        """Return a single item from the result without changing sort order.
+
+        @return: An arbitrary object or C{None} if one isn't available.
         """
         select = self._get_select()
         select.limit = 1
@@ -1062,20 +1093,21 @@ class ResultSet(object):
     def first(self):
         """Return the first item from an ordered result set.
 
-        Will raise UnorderedError if the result set isn't ordered.
-
-        See also last(), one(), and any().
+        @raises UnorderedError: Raised if the result set isn't ordered.
+        @return: The first object or C{None} if one isn't available.
+        @seealso: last(), one(), and any().
         """
         if self._order_by is Undef:
             raise UnorderedError("Can't use first() on unordered result set")
-        return self.any()
+        return self._any()
 
     def last(self):
         """Return the last item from an ordered result set.
 
-        Will raise UnorderedError if the result set isn't ordered.
-
-        See also first(), one(), and any().
+        @raises FeatureError: Raised if the result set has a C{LIMIT} set.
+        @raises UnorderedError: Raised if the result set isn't ordered.
+        @return: The last object or C{None} if one isn't available.
+        @seealso: first(), one(), and any().
         """
         if self._order_by is Undef:
             raise UnorderedError("Can't use last() on unordered result set")
@@ -1102,9 +1134,10 @@ class ResultSet(object):
     def one(self):
         """Return one item from a result set containing at most one item.
 
-        Will raise NotOneError if the result set contains more than one item.
-
-        See also first(), one(), and any().
+        @raises NotOneError: Raised if the result set contains more than one
+            item.
+        @return: The object or C{None} if one isn't available.
+        @seealso: first(), one(), and any().
         """
         select = self._get_select()
         # limit could be 1 due to slicing, for instance.
@@ -1233,6 +1266,27 @@ class ResultSet(object):
         """Get the sum of all values in an expression."""
         return self._aggregate(Sum, expr, expr)
 
+    def get_select_expr(self, *columns):
+        """Get a L{Select} expression to retrieve only the specified columns.
+
+        @param columns: One or more L{storm.expr.Column} objects whose values
+            will be fetched.
+        @raises FeatureError: Raised if no columns are specified or if this
+            result is a set expression such as a union.
+        @return: A L{Select} expression configured to use the query parameters
+            specified for this result set, and also limited to only retrieving
+            data for the specified columns.
+        """
+        if not columns:
+            raise FeatureError("select() takes at least one column "
+                               "as argument")
+        if self._select is not Undef:
+            raise FeatureError(
+                "Can't generate subselect expression for set expressions")
+        select = self._get_select()
+        select.columns = columns
+        return select
+
     def values(self, *columns):
         """Retrieve only the specified columns.
 
@@ -1240,12 +1294,16 @@ class ResultSet(object):
 
         @param columns: One or more L{storm.expr.Column} objects whose
             values will be fetched.
+        @raises FeatureError: Raised if no columns are specified or if this
+            result is a set expression such as a union.
         @return: An iterator of tuples of the values for each column
             from each matching row in the database.
         """
         if not columns:
             raise FeatureError("values() takes at least one column "
                                "as argument")
+        if self._select is not Undef:
+            raise FeatureError("values() can't be used with set expressions")
         select = self._get_select()
         select.columns = columns
         result = self._store._connection.execute(select)
@@ -1357,10 +1415,11 @@ class ResultSet(object):
             match = None
         else:
             match = compile_python.get_matcher(self._where)
+
             def get_column(column):
                 return obj_info.variables[column].get()
+
         objects = []
-        cls = self._find_spec.default_cls_info.cls
         for obj_info in self._store._iter_alive():
             try:
                 if (obj_info.cls_info is self._find_spec.default_cls_info and
@@ -1457,9 +1516,6 @@ class EmptyResultSet(object):
     def __init__(self, ordered=False):
         self._order_by = ordered
 
-    def _get_select(self):
-        return Select(SQLRaw("1"), SQLRaw("1 = 2"))
-
     def copy(self):
         result = EmptyResultSet(self._order_by)
         return result
@@ -1518,6 +1574,21 @@ class EmptyResultSet(object):
     def sum(self, column):
         return None
 
+    def get_select_expr(self, *columns):
+        """Get a L{Select} expression to retrieve only the specified columns.
+
+        @param columns: One or more L{storm.expr.Column} objects whose values
+            will be fetched.
+        @raises FeatureError: Raised if no columns are specified.
+        @return: A L{Select} expression configured to use the query parameters
+            specified for this result set.  The result of the select will
+            always be an empty set of rows.
+        """
+        if not columns:
+            raise FeatureError("select() takes at least one column "
+                               "as argument")
+        return Select(columns, False)
+
     def values(self, *columns):
         if not columns:
             raise FeatureError("values() takes at least one column "
