@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2006, 2007 Canonical
+# Copyright (c) 2006-2010 Canonical
 #
 # Written by Gustavo Niemeyer <gustavo@niemeyer.net>
 #
@@ -32,12 +32,12 @@ from storm.references import Reference, ReferenceSet
 from storm.properties import SimpleProperty, PropertyPublisherMeta
 from storm.variables import Variable
 from storm.exceptions import StormError, NotOneError
-from storm.info import get_cls_info, get_obj_info, ClassAlias
+from storm.info import get_cls_info, ClassAlias
 from storm.store import AutoReload, Store
 from storm.base import Storm
 from storm.expr import (
-    SQL, SQLRaw, Desc, And, Or, Not, In, Like, AutoTables, LeftJoin,
-    Column, compare_columns, Alias)
+    SQL, SQLRaw, Desc, And, Or, Not, In, Like, AutoTables, LeftJoin, Column,
+    compare_columns)
 from storm.tz import tzutc
 from storm import Undef
 
@@ -552,17 +552,22 @@ class SQLObjectResultSet(object):
         return item in result_set
 
     def __nonzero__(self):
+        """Return C{True} if this result set doesn't contain any results.
+
+        @note: This method is provided for compatibility with SQL Object.  For
+            new code, prefer L{is_empty}.  It's compatible with L{ResultSet}
+            which doesn't have a C{__nonzero__} implementation.
+        """
+        return not self.is_empty()
+
+    def is_empty(self):
+        """Return C{True} if this result set doesn't contain any results."""
         result_set = self._without_prejoins()._result_set
-        return not result_set.is_empty()
+        return result_set.is_empty()
 
     def count(self):
         result_set = self._without_prejoins()._result_set
-        # XXX 2008-04-14 jamesh: this should probably be handled at
-        # the store level.
-        if self._distinct:
-            return result_set.count(self._cls.id, distinct=True)
-        else:
-            return result_set.count()
+        return result_set.count()
 
     def orderBy(self, orderBy):
         return self._copy(orderBy=orderBy)
